/*! normalize.css v4.0.0 | MIT License | github.com/necolas/normalize.css */html {
 -ms-text-size-adjust:100%;
 -webkit-text-size-adjust:100%
}
body {
 margin:0
}
article,
aside,
details,
figcaption,
figure,
footer,
header,
main,
menu,
nav,
section,
summary {
 display:block
}
audio,
canvas,
progress,
video {
 display:inline-block
}
audio:not([controls]) {
 display:none;
 height:0
}
progress {
 vertical-align:baseline
}
template,
[hidden] {
 display:none
}
a {
 background-color:transparent
}
a:active,
a:hover,
a:focus {
 outline-width:0
}
abbr[title] {
 border-bottom:none;
 text-decoration:underline;
 text-decoration:underline dotted
}
b,
strong {
 font-weight:inherit
}
b,
strong {
 font-weight:bolder
}
dfn {
 font-style:italic
}
h1 {
 font-size:2em;
 margin:0.67em 0
}
mark {
 background-color:#ff0;
 color:#000
}
small {
 font-size:80%
}
sub,
sup {
 font-size:75%;
 line-height:0;
 position:relative;
 vertical-align:baseline
}
sub {
 bottom:-0.25em
}
sup {
 top:-0.5em
}
img {
 border-style:none
}
svg:not(:root) {
 overflow:hidden
}
code,
kbd,
pre,
samp {
 font-family:monospace, monospace;
 font-size:1em
}
figure {
 margin:0
}
hr {
 box-sizing:content-box;
 height:0;
 overflow:visible
}
button,
input,
select,
textarea {
 font:inherit
}
optgroup {
 font-weight:bold
}
button,
input,
select {
 overflow:visible
}
button,
input,
select,
textarea {
 margin:0
}
button,
select {
 text-transform:none
}
button,
[type="button"],
[type="reset"],
[type="submit"] {
 cursor:pointer
}
[disabled] {
 cursor:default
}
button,
html [type="button"],
[type="reset"],
[type="submit"] {
 -webkit-appearance:button
}
button::-moz-focus-inner,
input::-moz-focus-inner {
 border:0;
 outline:0;
 padding:0
}
button:-moz-focusring,
input:-moz-focusring {
 outline:1px dotted ButtonText
}
fieldset {
 border:1px solid #c0c0c0;
 margin:0 2px;
 padding:0.35em 0.625em 0.75em
}
legend {
 box-sizing:border-box;
 color:inherit;
 display:table;
 max-width:100%;
 padding:0;
 white-space:normal
}
textarea {
 overflow:auto
}
[type="checkbox"],
[type="radio"] {
 box-sizing:border-box;
 padding:0
}
[type="number"]::-webkit-inner-spin-button,
[type="number"]::-webkit-outer-spin-button {
 height:auto
}
[type="search"] {
 -webkit-appearance:textfield
}
[type="search"]::-webkit-search-cancel-button,
[type="search"]::-webkit-search-decoration {
 -webkit-appearance:none
}
html,
body {
 margin:0;
 padding:0;
 width:100%;
 height:100%;
 font-size:100%
}
body {
 font-family:Fira Sans,sans-serif;
 font-size:16px;
 font-size:1rem;
 color:#383838;
 background-color:#c3edf5;
 background-position:center center;
 background-attachment:fixed
}
@media only screen and (max-width: 30em) {
 body {
  background-image:none !important;
  background:#fff
 }
}
* {
 box-sizing:border-box
}
*::after {
 box-sizing:border-box
}
*::before {
 box-sizing:border-box
}
.header {
 height:100%;
 position:relative;
 width:100%;
 min-height:300px;
 text-align:left;
 color:#fff
}
@media only screen and (max-width: 30em) {
 .header {
  display:block !important
 }
}
@media only screen and (max-height: 500px) {
 .header {
  display:block !important
 }
}
.flex {
 display:-webkit-box;
 display:-webkit-flex;
 display:-ms-flexbox;
 display:flex;
 -webkit-box-flex:0;
 -webkit-flex:0 1 auto;
 -ms-flex:0 1 auto;
 flex:0 1 auto;
 -webkit-box-orient:horizontal;
 -webkit-box-direction:normal;
 -webkit-flex-direction:row;
 -ms-flex-direction:row;
 flex-direction:row;
 -webkit-flex-wrap:wrap;
 -ms-flex-wrap:wrap;
 flex-wrap:wrap;
 -webkit-box-align:center;
 -webkit-align-items:center;
 -ms-flex-align:center;
 align-items:center
}
.container {
 width:36rem;
 max-width:550px;
 padding-right:15px;
 padding-left:15px;
 display:block;
 margin-left:auto;
 margin-right:auto;
 background-color:#fff;
 color:#282828;
 z-index:1;
 border-radius:3px;
 box-shadow:0 0 10px 0 rgba(40,40,40,0.3);
 -webkit-transition:.5s;
 -moz-transition:.5s;
 -ms-transition:.5s;
 -o-transition:.5s;
 transition:.5s
}
@media only screen and (max-width: 30em) {
 .container {
  width:100%;
  box-shadow:none
 }
}
@media only screen and (min-width: 30.1em) and (max-width: 48em) {
 .container {
  width:80%
 }
}
.row {
 margin-top:15px;
 text-align:center
}
.row .title {
 margin-top:0;
 margin-bottom:5px
}
@media only screen and (max-width: 30em) {
 .row {
  padding:0 1em
 }
}
.wrapper {
 width:75%;
 max-width:850px;
 margin:4rem auto;
 background:#fff;
 color:#383838;
 padding:2em;
 border-radius:3px;
 box-shadow:0 0 10px 0 rgba(40,40,40,0.3);
 -webkit-transition:.5s;
 -moz-transition:.5s;
 -ms-transition:.5s;
 -o-transition:.5s;
 transition:.5s
}
@media only screen and (max-width: 30em) {
 .wrapper {
  width:90%;
  padding:2em 0;
  box-shadow:none
 }
}
#disqus_thread {
 width:75%;
 max-width:850px;
 margin:-3rem auto 4rem;
 background:#fff;
 color:#383838;
 padding:2em;
 border-radius:3px;
 box-shadow:0 0 10px 0 rgba(40,40,40,0.3);
 -webkit-transition:.5s;
 -moz-transition:.5s;
 -ms-transition:.5s;
 -o-transition:.5s;
 transition:.5s
}
@media only screen and (max-width: 30em) {
 #disqus_thread {
  width:90%;
  padding:2em 0;
  box-shadow:none
 }
}
.entry-meta {
 font-size:12px;
 font-size:.75rem;
 text-transform:uppercase;
 color:#d1d1d1;
 padding-left:0
}
.entry-meta a {
 color:#d1d1d1
}
.entry-meta li {
 list-style-type:none;
 display:inline
}
.entry-meta .tag {
 display:inline-block;
 border:1px solid;
 border-color:rgba(229,230,233,0.5) rgba(223,224,228,0.5) #d0d1d5;
 -webkit-border-radius:3px;
 -moz-border-radius:3px;
 border-radius:3px;
 text-decoration:none;
 background-color:#fff;
 color:#383838;
 -webkit-transition:.75s;
 -moz-transition:.75s;
 -ms-transition:.75s;
 -o-transition:.75s;
 transition:.75s;
 margin:2px 5px
}
.entry-meta .tag span {
 float:left;
 padding:2px 6px
}
.entry-meta .tag .count {
 background-color:#d9d9d9;
 -webkit-border-top-right-radius:3px;
 -webkit-border-bottom-right-radius:3px;
 -webkit-border-bottom-left-radius:0;
 -webkit-border-top-left-radius:0;
 -moz-border-radius-topright:3px;
 -moz-border-radius-bottomright:3px;
 -moz-border-radius-bottomleft:0;
 -moz-border-radius-topleft:0;
 border-top-right-radius:3px;
 border-bottom-right-radius:3px;
 border-bottom-left-radius:0;
 border-top-left-radius:0;
 -webkit-background-clip:padding-box;
 -moz-background-clip:padding;
 background-clip:padding-box
}
.entry-meta .tag:hover {
 background-color:#d9d9d9;
 box-shadow:0 0 1px 0 rgba(40,40,40,0.5)
}
header .entry-meta {
 display:none
}
@media only screen and (min-width: 48em) {
 header .entry-meta {
  display:block
 }
}
.entry-title a {
 text-decoration:none
}
.entry-tags {
 display:block;
 margin-top:1rem;
 width:70%;
 float:left;
 text-align:left
}
.social-share {
 display:block;
 margin-top:1rem;
 width:30%;
 float:right;
 text-align:right
}
.feature {
 -webkit-transition:.5s;
 -moz-transition:.5s;
 -ms-transition:.5s;
 -o-transition:.5s;
 transition:.5s
}
@media only screen and (max-width: 30em) {
 .feature {
  background-image:none !important
 }
}
@media only screen and (min-width: 30.1em) and (max-width: 48em) {
 .feature {
  color:white;
  padding:4em;
  margin-left:-4rem;
  margin-right:-4rem;
  margin-top:-2rem;
  border-radius:3px 3px 0 0;
  background-color:rgba(0,0,0,0.6);
  background-blend-mode:multiply;
  background-position:center center
 }
}
@media only screen and (min-width: 48em) {
 .feature {
  color:white;
  padding:10em;
  margin-left:-4rem;
  margin-right:-4rem;
  margin-top:-2rem;
  border-radius:3px 3px 0 0;
  background-color:rgba(0,0,0,0.6);
  background-blend-mode:multiply;
  background-position:center center
 }
}
.content {
 font-size:14px;
 font-size:.875rem;
 line-height:1.8571;
 margin-bottom:26px;
 margin-bottom:1.625rem;
 padding:0 2rem;
 -moz-osx-font-smoothing:grayscale;
 -webkit-font-smoothing:antialiased !important;
 -moz-font-smoothing:antialiased !important;
 text-rendering:optimizelegibility !important;
 text-align:justify
}
@media only screen and (max-width: 30em) {
 .content {
  padding:0 .5em
 }
}
.title {
 text-align:center;
 font-weight:bold
}
.title>p {
 text-align:center;
 font-weight:bold
}
.title>a>p {
 text-align:center;
 font-weight:bold
}
.post-title {
 text-align:center;
 font-weight:bold;
 margin-bottom:3rem
}
h3.title a {
 color:#282828;
 text-decoration:none
}
.post-list {
 text-align:left
}
.post-list ul {
 list-style:none;
 padding-left:0
}
.post-list li {
 padding:10px 18px;
 margin:0 10px;
 text-decoration:none;
 background:#fff;
 color:#282828
}
.post-list li a {
 font-size:20px;
 font-size:1.25rem;
 text-decoration:none;
 color:#282828;
 font-weight:bolder
}
.post-list li a:hover {
 color:#FF4136
}
.post-list li p {
 font-weight:lighter;
 font-size:16px;
 font-size:1rem
}
blockquote {
 font-family:serif;
 font-style:italic;
 border-left:8px solid #444;
 padding-left:20px;
 margin:0
}
@media only screen and (min-width: 48em) {
 blockquote {
  margin-left:-28px
 }
}
figcaption {
 padding-top:10px;
 font-size:14px;
 font-size:.875rem;
 line-height:1.8571;
 color:#8c8b8b
}
a {
 transition:color .4s;
 color:#282828
}
a:link {
 color:#282828
}
a:visited {
 color:#383838
}
a:hover {
 color:#FF4136
}
a:active {
 transition:color .3s;
 color:#85144b
}
pre,
code {
 font-family:Inconsolata;
 overflow-x:auto
}
p code,
li code,
p pre,
li pre {
 font-size:14px;
 font-size:.875rem;
 line-height:1.8571;
 background-color:#e8e8e8;
 border:1px solid #d0d1d5;
 -webkit-border-radius:4px;
 -moz-border-radius:4px;
 border-radius:4px;
 margin:0 2px;
 padding:0 5px
}
li .highlight code,
li .highlight pre {
 background-color:transparent;
 border:0
}
table {
 width:100%
}
.notice {
 margin-top:1.5em;
 padding:.5em 1em;
 text-indent:0;
 font-size:14px;
 font-size:.875rem;
 background-color:#e8e8e8;
 border:1px solid #bbb;
 -webkit-border-radius:3px;
 -moz-border-radius:3px;
 border-radius:3px
}
dt {
 font-weight:bolder
}
.social-btn,
.btn {
 display:inline-block;
 border:1px solid;
 border-color:rgba(229,230,233,0.5) rgba(223,224,228,0.5) #d0d1d5;
 border-radius:3px;
 text-decoration:none;
 background:#fff;
 color:#282828
}
.social-btn:hover,
.btn:hover {
 box-shadow:0 0 1px 0 rgba(40,40,40,0.5);
 color:#FF4136
}
.social-btn {
 padding:10px;
 margin:10px
}
.btn {
 padding:10px 18px;
 font-size:14px;
 font-size:.875rem;
 margin:4px auto
}
.btn.btn-success {
 background-color:#5cb85c;
 color:#fff;
 border-color:#5cb85c
}
.btn.btn-success:visited {
 color:#fff
}
.btn.btn-success:hover {
 background-color:#fff;
 color:#5cb85c
}
.btn.btn-warning {
 background-color:#dd8338;
 color:#fff;
 border-color:#dd8338
}
.btn.btn-warning:visited {
 color:#fff
}
.btn.btn-warning:hover {
 background-color:#fff;
 color:#dd8338
}
.btn.btn-danger {
 background-color:#C64537;
 color:#fff;
 border-color:#C64537
}
.btn.btn-danger:visited {
 color:#fff
}
.btn.btn-danger:hover {
 background-color:#fff;
 color:#C64537
}
.btn.btn-info {
 background-color:#308cbc;
 color:#fff;
 border-color:#308cbc
}
.btn.btn-info:visited {
 color:#fff
}
.btn.btn-info:hover {
 background-color:#fff;
 color:#308cbc
}
.title .btn {
 margin:10px;
 font-size:16px;
 font-size:1rem;
 font-weight:300
}
.post-list .btn {
 font-size:14px;
 font-size:.875rem
}
hr {
 border:0;
 height:1px;
 opacity:.8;
 margin:1em 0;
 background-image:-webkit-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-moz-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-ms-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-o-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0)
}
hr.hr-line {
 border:0;
 height:1px;
 opacity:.15;
 margin:1em 0;
 background-image:-webkit-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-moz-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-ms-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0);
 background-image:-o-linear-gradient(left, #f0f0f0, #8c8b8b, #f0f0f0)
}
img {
 display:block;
 max-width:100%;
 width:auto;
 height:auto;
 vertical-align:middle;
 margin-left:auto;
 margin-right:auto;
 border:0;
 -ms-interpolation-mode:bicubic;
 image-rendering:optimizeQuality
}
.img-circle {
 border-radius:50%;
 border:3px solid #fff;
 height:150px;
 width:150px
}
.author-photo {
 background-color:#fff
}
.image-right {
 float:right;
 margin-left:1em;
 margin-top:0
}
figure {
 margin:0;
 padding-top:10px;
 padding-bottom:10px;
 *zoom:1
}
figure:before,
figure:after {
 display:table;
 content:"";
 line-height:0
}
figure:after {
 clear:both
}
figure img {
 margin-bottom:10px;
 -webkit-border-radius:4px;
 -moz-border-radius:4px;
 border-radius:4px
}
@media only screen and (min-width: 62.5em) {
 figure.half img {
  width:calc(50% - 4px);
  float:left;
  margin-left:1px;
  margin-right:1px
 }
 figure.half figcaption {
  clear:left
 }
}
@media only screen and (min-width: 62.5em) {
 figure.third img {
  width:calc(33.3% - 6px);
  float:left;
  margin-left:1px;
  margin-right:1px
 }
 figure.third figcaption {
  clear:left
 }
}
svg:not(:root) {
 overflow:hidden
}
kbd {
 display:inline-block;
 margin:0 1px;
 padding:1px 8px;
 font-size:10px;
 font-size:.625rem;
 font-weight:bold;
 color:#555;
 vertical-align:middle;
 background-color:#fcfcfc;
 border:solid 1px #ccc;
 border-bottom-color:#bbb;
 border-radius:3px;
 box-shadow:inset 0 -1px 0 #bbb
}
.goup-container {
 border-radius:3px !important;
 border-color:rgba(229,230,233,0.5) rgba(223,224,228,0.5) #d0d1d5 !important;
 border:1px solid
}
@media only screen and (max-width: 30em) {
 .goup-container {
  display:none !important
 }
}
.goup-arrow {
 border-color:transparent transparent #383838 !important
}
.reading-time {
 text-align:center;
 font-weight:300;
 margin-bottom:10px
}
pre.highlight {
 padding:1em
}
.highlight {
 background-color:#49483e;
 font-family:Inconsolata;
 font-size:14px;
 font-size:.875rem;
 line-height:1.8571;
 margin-bottom:26px;
 margin-bottom:1.625rem;
 color:#f8f8f2;
 border-radius:4px
}
.highlight pre {
 position:relative;
 margin:0;
 padding:1em
}
.highlight .lineno {
 padding-right:24px;
 color:#c8c7be
}
.highlight .hll {
 background-color:#49483e
}
.highlight .c {
 color:#75715e
}
.highlight .err {
 color:#960050;
 background-color:#1e0010
}
.highlight .k {
 color:#66d9ef
}
.highlight .l {
 color:#ae81ff
}
.highlight .n {
 color:#f8f8f2
}
.highlight .o {
 color:#f92672
}
.highlight .p {
 color:#f8f8f2
}
.highlight .cm {
 color:#75715e
}
.highlight .cp {
 color:#75715e
}
.highlight .c1 {
 color:#75715e
}
.highlight .cs {
 color:#75715e
}
.highlight .ge {
 font-style:italic
}
.highlight .gs {
 font-weight:bold
}
.highlight .kc {
 color:#66d9ef
}
.highlight .kd {
 color:#66d9ef
}
.highlight .kn {
 color:#f92672
}
.highlight .kp {
 color:#66d9ef
}
.highlight .kr {
 color:#66d9ef
}
.highlight .kt {
 color:#66d9ef
}
.highlight .ld {
 color:#e6db74
}
.highlight .m {
 color:#ae81ff
}
.highlight .s {
 color:#e6db74
}
.highlight .na {
 color:#a6e22e
}
.highlight .nb {
 color:#f8f8f2
}
.highlight .nc {
 color:#a6e22e
}
.highlight .no {
 color:#66d9ef
}
.highlight .nd {
 color:#a6e22e
}
.highlight .ni {
 color:#f8f8f2
}
.highlight .ne {
 color:#a6e22e
}
.highlight .nf {
 color:#a6e22e
}
.highlight .nl {
 color:#f8f8f2
}
.highlight .nn {
 color:#f8f8f2
}
.highlight .nx {
 color:#a6e22e
}
.highlight .py {
 color:#f8f8f2
}
.highlight .nt {
 color:#f92672
}
.highlight .nv {
 color:#f8f8f2
}
.highlight .ow {
 color:#f92672
}
.highlight .w {
 color:#f8f8f2
}
.highlight .mf {
 color:#ae81ff
}
.highlight .mh {
 color:#ae81ff
}
.highlight .mi {
 color:#ae81ff
}
.highlight .mo {
 color:#ae81ff
}
.highlight .sb {
 color:#e6db74
}
.highlight .sc {
 color:#e6db74
}
.highlight .sd {
 color:#e6db74
}
.highlight .s2 {
 color:#e6db74
}
.highlight .se {
 color:#ae81ff
}
.highlight .sh {
 color:#e6db74
}
.highlight .si {
 color:#e6db74
}
.highlight .sx {
 color:#e6db74
}
.highlight .sr {
 color:#e6db74
}
.highlight .s1 {
 color:#e6db74
}
.highlight .ss {
 color:#e6db74
}
.highlight .bp {
 color:#f8f8f2
}
.highlight .vc {
 color:#f8f8f2
}
.highlight .vg {
 color:#f8f8f2
}
.highlight .vi {
 color:#f8f8f2
}
.highlight .il {
 color:#ae81ff
}
.dl-menuwrapper {
 width:100%;
 position:absolute;
 top:0;
 left:0;
 z-index:1000;
 -webkit-perspective:1000px;
 -moz-perspective:1000px;
 perspective:1000px;
 -webkit-perspective-origin:50% 200%;
 -moz-perspective-origin:50% 200%;
 perspective-origin:50% 200%
}
@media only screen and (min-width: 48em) {
 .dl-menuwrapper {
  position:fixed;
  max-width:175px;
  top:25px;
  left:25px
 }
}
.dl-menuwrapper button {
 top:0;
 left:0;
 background:#222;
 border:none;
 width:48px;
 height:45px;
 text-indent:-900em;
 overflow:hidden;
 position:relative;
 cursor:pointer;
 outline:none;
 -webkit-border-top-right-radius:0;
 -webkit-border-bottom-right-radius:3px;
 -webkit-border-bottom-left-radius:0;
 -webkit-border-top-left-radius:0;
 -moz-border-radius-topright:0;
 -moz-border-radius-bottomright:3px;
 -moz-border-radius-bottomleft:0;
 -moz-border-radius-topleft:0;
 border-top-right-radius:0;
 border-bottom-right-radius:3px;
 border-bottom-left-radius:0;
 border-top-left-radius:0;
 -webkit-background-clip:padding-box;
 -moz-background-clip:padding;
 background-clip:padding-box;
 opacity:0.6
}
@media only screen and (min-width: 48em) {
 .dl-menuwrapper button {
  -webkit-border-top-right-radius:3px;
  -webkit-border-bottom-right-radius:3px;
  -webkit-border-bottom-left-radius:3px;
  -webkit-border-top-left-radius:3px;
  -moz-border-radius-topright:3px;
  -moz-border-radius-bottomright:3px;
  -moz-border-radius-bottomleft:3px;
  -moz-border-radius-topleft:3px;
  border-top-right-radius:3px;
  border-bottom-right-radius:3px;
  border-bottom-left-radius:3px;
  border-top-left-radius:3px;
  -webkit-background-clip:padding-box;
  -moz-background-clip:padding;
  background-clip:padding-box
 }
}
.dl-menuwrapper button:hover,
.dl-menuwrapper button.dl-active,
.dl-menuwrapper ul {
 background:#aaa
}
.dl-menuwrapper button:after {
 content:'';
 position:absolute;
 width:68%;
 height:5px;
 background:#fff;
 top:10px;
 left:16%;
 box-shadow:0 10px 0 #fff,0 20px 0 #fff
}
.dl-menuwrapper button.dl-active {
 display:none
}
.dl-menuwrapper ul {
 padding:0;
 list-style:none;
 -webkit-transform-style:preserve-3d;
 -moz-transform-style:preserve-3d;
 transform-style:preserve-3d
}
.dl-menuwrapper li {
 position:relative
}
.dl-menuwrapper li h4 {
 margin:0;
 padding:15px 20px 0;
 color:rgba(255,255,255,0.9)
}
.dl-menuwrapper li p {
 margin:0;
 padding:15px 20px;
 font-size:14px;
 font-size:.875rem;
 color:rgba(255,255,255,0.8);
 font-weight:300
}
.dl-menuwrapper li p a {
 display:inline;
 padding:0;
 font-size:14px;
 font-size:.875rem
}
.dl-menuwrapper li a {
 display:block;
 position:relative;
 padding:10px 20px;
 font-size:14px;
 font-size:.875rem;
 line-height:20px;
 font-weight:400;
 color:#fff;
 outline:none;
 text-decoration:none
}
.dl-menuwrapper li.dl-back>a {
 padding-left:30px;
 background:rgba(0,0,0,0.2)
}
.dl-menuwrapper li.dl-back:after,
.dl-menuwrapper li>a:not(:only-child):after {
 position:absolute;
 top:0;
 line-height:40px;
 font-family:'fontawesome';
 color:#fff;
 speak:none;
 -webkit-font-smoothing:antialiased;
 content:"\f105"
}
.dl-menuwrapper li.dl-back:after {
 left:10px;
 color:rgba(212,204,198,0.5);
 -webkit-transform:rotate(180deg);
 -moz-transform:rotate(180deg);
 transform:rotate(180deg)
}
.dl-menuwrapper li>a:after {
 right:10px;
 color:rgba(0,0,0,0.15)
}
.dl-menuwrapper .dl-menu {
 margin:0;
 position:absolute;
 width:100%;
 max-width:400px;
 overflow-y:auto;
 max-height:600px;
 opacity:0;
 pointer-events:none;
 box-shadow:0 12px 24px rgba(0,0,0,0.4);
 -webkit-transform:translateY(10px);
 -moz-transform:translateY(10px);
 transform:translateY(10px);
 -webkit-backface-visibility:hidden;
 -moz-backface-visibility:hidden;
 backface-visibility:hidden
}
@media only screen and (min-width: 48em) {
 .dl-menuwrapper .dl-menu {
  -webkit-border-top-right-radius:3px;
  -webkit-border-bottom-right-radius:3px;
  -webkit-border-bottom-left-radius:3px;
  -webkit-border-top-left-radius:3px;
  -moz-border-radius-topright:3px;
  -moz-border-radius-bottomright:3px;
  -moz-border-radius-bottomleft:3px;
  -moz-border-radius-topleft:3px;
  border-top-right-radius:3px;
  border-bottom-right-radius:3px;
  border-bottom-left-radius:3px;
  border-top-left-radius:3px;
  -webkit-background-clip:padding-box;
  -moz-background-clip:padding;
  background-clip:padding-box;
  max-height:650px
 }
}
.dl-menuwrapper .dl-menu.dl-menu-toggle {
 -webkit-transition:all 0.3s ease;
 -moz-transition:all 0.3s ease;
 transition:all 0.3s ease
}
.dl-menuwrapper .dl-menu.dl-menuopen {
 opacity:1;
 pointer-events:auto;
 -webkit-transform:translateY(0px);
 -moz-transform:translateY(0px);
 transform:translateY(0px)
}
.dl-menuwrapper .dl-submenu {
 -webkit-border-radius:3px;
 -moz-border-radius:3px;
 border-radius:3px;
 box-shadow:0 12px 24px rgba(0,0,0,0.4)
}
.dl-menuwrapper .dl-submenu .btn {
 margin-bottom:0
}
.dl-menuwrapper li .dl-submenu {
 display:none
}
.dl-menu.dl-subview li,
.dl-menu.dl-subview li.dl-subviewopen>a,
.dl-menu.dl-subview li.dl-subview>a {
 display:none
}
.dl-menu.dl-subview li.dl-subview,
.dl-menu.dl-subview li.dl-subview .dl-submenu,
.dl-menu.dl-subview li.dl-subviewopen,
.dl-menu.dl-subview li.dl-subviewopen>.dl-submenu,
.dl-menu.dl-subview li.dl-subviewopen>.dl-submenu>li {
 display:block
}
.dl-menu.dl-animate-out {
 -webkit-animation:MenuAnimOut 0.4s ease;
 -moz-animation:MenuAnimOut 0.4s ease;
 animation:MenuAnimOut 0.4s ease
}
@-webkit-keyframes MenuAnimOut {
 100% {
  -webkit-transform:translateZ(300px);
  opacity:0
 }
}
@-moz-keyframes MenuAnimOut {
 100% {
  -moz-transform:translateZ(300px);
  opacity:0
 }
}
@keyframes MenuAnimOut {
 100% {
  transform:translateZ(300px);
  opacity:0
 }
}
.dl-menu.dl-animate-in {
 -webkit-animation:MenuAnimIn 0.4s ease;
 -moz-animation:MenuAnimIn 0.4s ease;
 animation:MenuAnimIn 0.4s ease
}
@-webkit-keyframes MenuAnimIn {
 0% {
  -webkit-transform:translateZ(300px);
  opacity:0
 }
 100% {
  -webkit-transform:translateZ(0px);
  opacity:1
 }
}
@-moz-keyframes MenuAnimIn {
 0% {
  -moz-transform:translateZ(300px);
  opacity:0
 }
 100% {
  -moz-transform:translateZ(0px);
  opacity:1
 }
}
@keyframes MenuAnimIn {
 0% {
  transform:translateZ(300px);
  opacity:0
 }
 100% {
  transform:translateZ(0px);
  opacity:1
 }
}
.dl-menuwrapper>.dl-submenu.dl-animate-in {
 -webkit-animation:SubMenuAnimIn 0.4s ease;
 -moz-animation:SubMenuAnimIn 0.4s ease;
 animation:SubMenuAnimIn 0.4s ease
}
@-webkit-keyframes SubMenuAnimIn {
 0% {
  -webkit-transform:translateZ(-300px);
  opacity:0
 }
 100% {
  -webkit-transform:translateZ(0px);
  opacity:1
 }
}
@-moz-keyframes SubMenuAnimIn {
 0% {
  -moz-transform:translateZ(-300px);
  opacity:0
 }
 100% {
  -moz-transform:translateZ(0px);
  opacity:1
 }
}
@keyframes SubMenuAnimIn {
 0% {
  transform:translateZ(-300px);
  opacity:0
 }
 100% {
  transform:translateZ(0px);
  opacity:1
 }
}
.dl-menuwrapper>.dl-submenu.dl-animate-out {
 -webkit-animation:SubMenuAnimOut 0.4s ease;
 -moz-animation:SubMenuAnimOut 0.4s ease;
 animation:SubMenuAnimOut 0.4s ease
}
@-webkit-keyframes SubMenuAnimOut {
 0% {
  -webkit-transform:translateZ(0px);
  opacity:1
 }
 100% {
  -webkit-transform:translateZ(-300px);
  opacity:0
 }
}
@-moz-keyframes SubMenuAnimOut {
 0% {
  -moz-transform:translateZ(0px);
  opacity:1
 }
 100% {
  -moz-transform:translateZ(-300px);
  opacity:0
 }
}
@keyframes SubMenuAnimOut {
 0% {
  transform:translateZ(0px);
  opacity:1
 }
 100% {
  transform:translateZ(-300px);
  opacity:0
 }
}
.no-touchevents .dl-menuwrapper li a:hover {
 background:rgba(255,248,213,0.1)
}
.no-js .dl-trigger {
 display:none
}
.no-js .dl-menuwrapper {
 position:initial
}
@media only screen and (min-width: 48em) {
 .no-js .dl-menuwrapper {
  position:absolute
 }
}
.no-js .dl-menuwrapper .dl-menu {
 position:relative;
 opacity:1;
 pointer-events:auto;
 -webkit-transform:none;
 -moz-transform:none;
 transform:none
}
.no-js .dl-menuwrapper li .dl-submenu {
 display:block
}
.no-js .dl-menuwrapper li.dl-back {
 display:none
}
.no-js .dl-menuwrapper li>a:not(:only-child) {
 background:rgba(0,0,0,0.1)
}
.no-js .dl-menuwrapper li>a:not(:only-child):after {
 content:''
}
.no-js .dl-menu {
 max-height:100%
}
.no-js .dl-menu li {
 display:block
}
.dl-menuwrapper button:hover,
.dl-menuwrapper button.dl-active,
.dl-menuwrapper ul {
 background:#222
}
.dl-menu li {
 display:none
}
.dl-menuopen li {
 display:block
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-Light.eot");
 src:local("Fira Sans Light"),
 url("../fonts/FiraSans/eot/FiraSans-Light.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-Light.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-Light.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-Light.ttf") format("truetype");
 font-weight:300;
 font-style:normal
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-LightItalic.eot");
 src:local("Fira Sans Light Italic"),
 url("../fonts/FiraSans/eot/FiraSans-LightItalic.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-LightItalic.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-LightItalic.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-LightItalic.ttf") format("truetype");
 font-weight:300;
 font-style:italic
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-Regular.eot");
 src:local("Fira Sans Regular"),
 url("../fonts/FiraSans/eot/FiraSans-Regular.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-Regular.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-Regular.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-Regular.ttf") format("truetype");
 font-weight:400;
 font-style:normal
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-Italic.eot");
 src:local("Fira Sans Regular Italic"),
 url("../fonts/FiraSans/eot/FiraSans-Italic.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-Italic.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-Italic.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-Italic.ttf") format("truetype");
 font-weight:400;
 font-style:italic
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-Medium.eot");
 src:local("Fira Sans Medium"),
 url("../fonts/FiraSans/eot/FiraSans-Medium.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-Medium.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-Medium.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-Medium.ttf") format("truetype");
 font-weight:500;
 font-style:normal
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-MediumItalic.eot");
 src:local("Fira Sans Medium Italic"),
 url("../fonts/FiraSans/eot/FiraSans-MediumItalic.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-MediumItalic.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-MediumItalic.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-MediumItalic.ttf") format("truetype");
 font-weight:500;
 font-style:italic
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-Bold.eot");
 src:local("Fira Sans Bold"),
 url("../fonts/FiraSans/eot/FiraSans-Bold.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-Bold.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-Bold.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-Bold.ttf") format("truetype");
 font-weight:600;
 font-style:normal
}
@font-face {
 font-family:'Fira Sans';
 src:url("../fonts/FiraSans/eot/FiraSans-BoldItalic.eot");
 src:local("Fira Sans Bold Italic"),
 url("../fonts/FiraSans/eot/FiraSans-BoldItalic.eot") format("embedded-opentype"),
 url("../fonts/FiraSans/woff/FiraSans-BoldItalic.woff") format("woff"),
 url("../fonts/FiraSans/woff2/FiraSans-BoldItalic.woff2") format("woff2"),
 url("../fonts/FiraSans/ttf/FiraSans-BoldItalic.ttf") format("truetype");
 font-weight:600;
 font-style:italic
}
@font-face {
 font-family:'Inconsolata';
 font-style:normal;
 font-weight:400;
 src:local("Inconsolata"),
 url(https://fonts.gstatic.com/s/inconsolata/v12/BjAYBlHtW3CJxDcjzrnZCCYE0-AqJ3nfInTTiDXDjU4.woff2) format("woff2");
 unicode-range:U+0100-024F,
 U+1E00-1EFF,
 U+20A0-20AB,
 U+20AD-20CF,
 U+2C60-2C7F,
 U+A720-A7FF
}
@font-face {
 font-family:'Inconsolata';
 font-style:normal;
 font-weight:400;
 src:local("Inconsolata"),
 url(https://fonts.gstatic.com/s/inconsolata/v12/BjAYBlHtW3CJxDcjzrnZCI4P5ICox8Kq3LLUNMylGO4.woff2) format("woff2");
 unicode-range:U+0000-00FF,
 U+0131,
 U+0152-0153,
 U+02C6,
 U+02DA,
 U+02DC,
 U+2000-206F,
 U+2074,
 U+20AC,
 U+2212,
 U+2215,
 U+E0FF,
 U+EFFD,
 U+F000
}
@font-face {
 font-family:'Inconsolata';
 font-style:normal;
 font-weight:700;
 src:local("Inconsolata Bold"),
 local("Inconsolata-Bold"),
 url(https://fonts.gstatic.com/s/inconsolata/v12/AIed271kqQlcIRSOnQH0yejkDdvhIIFj_YMdgqpnSB0.woff2) format("woff2");
 unicode-range:U+0100-024F,
 U+1E00-1EFF,
 U+20A0-20AB,
 U+20AD-20CF,
 U+2C60-2C7F,
 U+A720-A7FF
}
@font-face {
 font-family:'Inconsolata';
 font-style:normal;
 font-weight:700;
 src:local("Inconsolata Bold"),
 local("Inconsolata-Bold"),
 url(https://fonts.gstatic.com/s/inconsolata/v12/AIed271kqQlcIRSOnQH0yYlIZu-HDpmDIZMigmsroc4.woff2) format("woff2");
 unicode-range:U+0000-00FF,
 U+0131,
 U+0152-0153,
 U+02C6,
 U+02DA,
 U+02DC,
 U+2000-206F,
 U+2074,
 U+20AC,
 U+2212,
 U+2215,
 U+E0FF,
 U+EFFD,
 U+F000
}
/*!
 *  Font Awesome 4.7.0 by @davegandy - http://fontawesome.io - @fontawesome
 *  License - http://fontawesome.io/license (Font: SIL OFL 1.1, CSS: MIT License)
 */@font-face {
 font-family:'FontAwesome';
 src:url("../fonts/FontAwesome//fontawesome-webfont.eot?v=4.7.0");
 src:url("../fonts/FontAwesome//fontawesome-webfont.eot?#iefix&v=4.7.0") format("embedded-opentype"),
 url("../fonts/FontAwesome//fontawesome-webfont.woff2?v=4.7.0") format("woff2"),
 url("../fonts/FontAwesome//fontawesome-webfont.woff?v=4.7.0") format("woff"),
 url("../fonts/FontAwesome//fontawesome-webfont.ttf?v=4.7.0") format("truetype"),
 url("../fonts/FontAwesome//fontawesome-webfont.svg?v=4.7.0#fontawesomeregular") format("svg");
 font-weight:normal;
 font-style:normal
}
.fa {
 display:inline-block;
 font:normal normal normal 14px/1 FontAwesome;
 font-size:inherit;
 text-rendering:auto;
 -webkit-font-smoothing:antialiased;
 -moz-osx-font-smoothing:grayscale
}
.fa-lg {
 font-size:1.3333333333em;
 line-height:.75em;
 vertical-align:-15%
}
.fa-2x {
 font-size:2em
}
.fa-3x {
 font-size:3em
}
.fa-4x {
 font-size:4em
}
.fa-5x {
 font-size:5em
}
.fa-fw {
 width:1.2857142857em;
 text-align:center
}
.fa-ul {
 padding-left:0;
 margin-left:2.1428571429em;
 list-style-type:none
}
.fa-ul>li {
 position:relative
}
.fa-li {
 position:absolute;
 left:-2.1428571429em;
 width:2.1428571429em;
 top:.1428571429em;
 text-align:center
}
.fa-li.fa-lg {
 left:-1.8571428571em
}
.fa-border {
 padding:.2em .25em .15em;
 border:solid 0.08em #eee;
 border-radius:.1em
}
.fa-pull-left {
 float:left
}
.fa-pull-right {
 float:right
}
.fa.fa-pull-left {
 margin-right:.3em
}
.fa.fa-pull-right {
 margin-left:.3em
}
.pull-right {
 float:right
}
.pull-left {
 float:left
}
.fa.pull-left {
 margin-right:.3em
}
.fa.pull-right {
 margin-left:.3em
}
.fa-spin {
 -webkit-animation:fa-spin 2s infinite linear;
 animation:fa-spin 2s infinite linear
}
.fa-pulse {
 -webkit-animation:fa-spin 1s infinite steps(8);
 animation:fa-spin 1s infinite steps(8)
}
@-webkit-keyframes fa-spin {
 0% {
  -webkit-transform:rotate(0deg);
  transform:rotate(0deg)
 }
 100% {
  -webkit-transform:rotate(359deg);
  transform:rotate(359deg)
 }
}
@keyframes fa-spin {
 0% {
  -webkit-transform:rotate(0deg);
  transform:rotate(0deg)
 }
 100% {
  -webkit-transform:rotate(359deg);
  transform:rotate(359deg)
 }
}
.fa-rotate-90 {
 -ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=1)";
 -webkit-transform:rotate(90deg);
 -ms-transform:rotate(90deg);
 transform:rotate(90deg)
}
.fa-rotate-180 {
 -ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=2)";
 -webkit-transform:rotate(180deg);
 -ms-transform:rotate(180deg);
 transform:rotate(180deg)
}
.fa-rotate-270 {
 -ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=3)";
 -webkit-transform:rotate(270deg);
 -ms-transform:rotate(270deg);
 transform:rotate(270deg)
}
.fa-flip-horizontal {
 -ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=0, mirror=1)";
 -webkit-transform:scale(-1, 1);
 -ms-transform:scale(-1, 1);
 transform:scale(-1, 1)
}
.fa-flip-vertical {
 -ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=2, mirror=1)";
 -webkit-transform:scale(1, -1);
 -ms-transform:scale(1, -1);
 transform:scale(1, -1)
}
:root .fa-rotate-90,
:root .fa-rotate-180,
:root .fa-rotate-270,
:root .fa-flip-horizontal,
:root .fa-flip-vertical {
 filter:none
}
.fa-stack {
 position:relative;
 display:inline-block;
 width:2em;
 height:2em;
 line-height:2em;
 vertical-align:middle
}
.fa-stack-1x,
.fa-stack-2x {
 position:absolute;
 left:0;
 width:100%;
 text-align:center
}
.fa-stack-1x {
 line-height:inherit
}
.fa-stack-2x {
 font-size:2em
}
.fa-inverse {
 color:#fff
}
.fa-glass:before {
 content:""
}
.fa-music:before {
 content:""
}
.fa-search:before {
 content:""
}
.fa-envelope-o:before {
 content:""
}
.fa-heart:before {
 content:""
}
.fa-star:before {
 content:""
}
.fa-star-o:before {
 content:""
}
.fa-user:before {
 content:""
}
.fa-film:before {
 content:""
}
.fa-th-large:before {
 content:""
}
.fa-th:before {
 content:""
}
.fa-th-list:before {
 content:""
}
.fa-check:before {
 content:""
}
.fa-remove:before,
.fa-close:before,
.fa-times:before {
 content:""
}
.fa-search-plus:before {
 content:""
}
.fa-search-minus:before {
 content:""
}
.fa-power-off:before {
 content:""
}
.fa-signal:before {
 content:""
}
.fa-gear:before,
.fa-cog:before {
 content:""
}
.fa-trash-o:before {
 content:""
}
.fa-home:before {
 content:""
}
.fa-file-o:before {
 content:""
}
.fa-clock-o:before {
 content:""
}
.fa-road:before {
 content:""
}
.fa-download:before {
 content:""
}
.fa-arrow-circle-o-down:before {
 content:""
}
.fa-arrow-circle-o-up:before {
 content:""
}
.fa-inbox:before {
 content:""
}
.fa-play-circle-o:before {
 content:""
}
.fa-rotate-right:before,
.fa-repeat:before {
 content:""
}
.fa-refresh:before {
 content:""
}
.fa-list-alt:before {
 content:""
}
.fa-lock:before {
 content:""
}
.fa-flag:before {
 content:""
}
.fa-headphones:before {
 content:""
}
.fa-volume-off:before {
 content:""
}
.fa-volume-down:before {
 content:""
}
.fa-volume-up:before {
 content:""
}
.fa-qrcode:before {
 content:""
}
.fa-barcode:before {
 content:""
}
.fa-tag:before {
 content:""
}
.fa-tags:before {
 content:""
}
.fa-book:before {
 content:""
}
.fa-bookmark:before {
 content:""
}
.fa-print:before {
 content:""
}
.fa-camera:before {
 content:""
}
.fa-font:before {
 content:""
}
.fa-bold:before {
 content:""
}
.fa-italic:before {
 content:""
}
.fa-text-height:before {
 content:""
}
.fa-text-width:before {
 content:""
}
.fa-align-left:before {
 content:""
}
.fa-align-center:before {
 content:""
}
.fa-align-right:before {
 content:""
}
.fa-align-justify:before {
 content:""
}
.fa-list:before {
 content:""
}
.fa-dedent:before,
.fa-outdent:before {
 content:""
}
.fa-indent:before {
 content:""
}
.fa-video-camera:before {
 content:""
}
.fa-photo:before,
.fa-image:before,
.fa-picture-o:before {
 content:""
}
.fa-pencil:before {
 content:""
}
.fa-map-marker:before {
 content:""
}
.fa-adjust:before {
 content:""
}
.fa-tint:before {
 content:""
}
.fa-edit:before,
.fa-pencil-square-o:before {
 content:""
}
.fa-share-square-o:before {
 content:""
}
.fa-check-square-o:before {
 content:""
}
.fa-arrows:before {
 content:""
}
.fa-step-backward:before {
 content:""
}
.fa-fast-backward:before {
 content:""
}
.fa-backward:before {
 content:""
}
.fa-play:before {
 content:""
}
.fa-pause:before {
 content:""
}
.fa-stop:before {
 content:""
}
.fa-forward:before {
 content:""
}
.fa-fast-forward:before {
 content:""
}
.fa-step-forward:before {
 content:""
}
.fa-eject:before {
 content:""
}
.fa-chevron-left:before {
 content:""
}
.fa-chevron-right:before {
 content:""
}
.fa-plus-circle:before {
 content:""
}
.fa-minus-circle:before {
 content:""
}
.fa-times-circle:before {
 content:""
}
.fa-check-circle:before {
 content:""
}
.fa-question-circle:before {
 content:""
}
.fa-info-circle:before {
 content:""
}
.fa-crosshairs:before {
 content:""
}
.fa-times-circle-o:before {
 content:""
}
.fa-check-circle-o:before {
 content:""
}
.fa-ban:before {
 content:""
}
.fa-arrow-left:before {
 content:""
}
.fa-arrow-right:before {
 content:""
}
.fa-arrow-up:before {
 content:""
}
.fa-arrow-down:before {
 content:""
}
.fa-mail-forward:before,
.fa-share:before {
 content:""
}
.fa-expand:before {
 content:""
}
.fa-compress:before {
 content:""
}
.fa-plus:before {
 content:""
}
.fa-minus:before {
 content:""
}
.fa-asterisk:before {
 content:""
}
.fa-exclamation-circle:before {
 content:""
}
.fa-gift:before {
 content:""
}
.fa-leaf:before {
 content:""
}
.fa-fire:before {
 content:""
}
.fa-eye:before {
 content:""
}
.fa-eye-slash:before {
 content:""
}
.fa-warning:before,
.fa-exclamation-triangle:before {
 content:""
}
.fa-plane:before {
 content:""
}
.fa-calendar:before {
 content:""
}
.fa-random:before {
 content:""
}
.fa-comment:before {
 content:""
}
.fa-magnet:before {
 content:""
}
.fa-chevron-up:before {
 content:""
}
.fa-chevron-down:before {
 content:""
}
.fa-retweet:before {
 content:""
}
.fa-shopping-cart:before {
 content:""
}
.fa-folder:before {
 content:""
}
.fa-folder-open:before {
 content:""
}
.fa-arrows-v:before {
 content:""
}
.fa-arrows-h:before {
 content:""
}
.fa-bar-chart-o:before,
.fa-bar-chart:before {
 content:""
}
.fa-twitter-square:before {
 content:""
}
.fa-facebook-square:before {
 content:""
}
.fa-camera-retro:before {
 content:""
}
.fa-key:before {
 content:""
}
.fa-gears:before,
.fa-cogs:before {
 content:""
}
.fa-comments:before {
 content:""
}
.fa-thumbs-o-up:before {
 content:""
}
.fa-thumbs-o-down:before {
 content:""
}
.fa-star-half:before {
 content:""
}
.fa-heart-o:before {
 content:""
}
.fa-sign-out:before {
 content:""
}
.fa-linkedin-square:before {
 content:""
}
.fa-thumb-tack:before {
 content:""
}
.fa-external-link:before {
 content:""
}
.fa-sign-in:before {
 content:""
}
.fa-trophy:before {
 content:""
}
.fa-github-square:before {
 content:""
}
.fa-upload:before {
 content:""
}
.fa-lemon-o:before {
 content:""
}
.fa-phone:before {
 content:""
}
.fa-square-o:before {
 content:""
}
.fa-bookmark-o:before {
 content:""
}
.fa-phone-square:before {
 content:""
}
.fa-twitter:before {
 content:""
}
.fa-facebook-f:before,
.fa-facebook:before {
 content:""
}
.fa-github:before {
 content:""
}
.fa-unlock:before {
 content:""
}
.fa-credit-card:before {
 content:""
}
.fa-feed:before,
.fa-rss:before {
 content:""
}
.fa-hdd-o:before {
 content:""
}
.fa-bullhorn:before {
 content:""
}
.fa-bell:before {
 content:""
}
.fa-certificate:before {
 content:""
}
.fa-hand-o-right:before {
 content:""
}
.fa-hand-o-left:before {
 content:""
}
.fa-hand-o-up:before {
 content:""
}
.fa-hand-o-down:before {
 content:""
}
.fa-arrow-circle-left:before {
 content:""
}
.fa-arrow-circle-right:before {
 content:""
}
.fa-arrow-circle-up:before {
 content:""
}
.fa-arrow-circle-down:before {
 content:""
}
.fa-globe:before {
 content:""
}
.fa-wrench:before {
 content:""
}
.fa-tasks:before {
 content:""
}
.fa-filter:before {
 content:""
}
.fa-briefcase:before {
 content:""
}
.fa-arrows-alt:before {
 content:""
}
.fa-group:before,
.fa-users:before {
 content:""
}
.fa-chain:before,
.fa-link:before {
 content:""
}
.fa-cloud:before {
 content:""
}
.fa-flask:before {
 content:""
}
.fa-cut:before,
.fa-scissors:before {
 content:""
}
.fa-copy:before,
.fa-files-o:before {
 content:""
}
.fa-paperclip:before {
 content:""
}
.fa-save:before,
.fa-floppy-o:before {
 content:""
}
.fa-square:before {
 content:""
}
.fa-navicon:before,
.fa-reorder:before,
.fa-bars:before {
 content:""
}
.fa-list-ul:before {
 content:""
}
.fa-list-ol:before {
 content:""
}
.fa-strikethrough:before {
 content:""
}
.fa-underline:before {
 content:""
}
.fa-table:before {
 content:""
}
.fa-magic:before {
 content:""
}
.fa-truck:before {
 content:""
}
.fa-pinterest:before {
 content:""
}
.fa-pinterest-square:before {
 content:""
}
.fa-google-plus-square:before {
 content:""
}
.fa-google-plus:before {
 content:""
}
.fa-money:before {
 content:""
}
.fa-caret-down:before {
 content:""
}
.fa-caret-up:before {
 content:""
}
.fa-caret-left:before {
 content:""
}
.fa-caret-right:before {
 content:""
}
.fa-columns:before {
 content:""
}
.fa-unsorted:before,
.fa-sort:before {
 content:""
}
.fa-sort-down:before,
.fa-sort-desc:before {
 content:""
}
.fa-sort-up:before,
.fa-sort-asc:before {
 content:""
}
.fa-envelope:before {
 content:""
}
.fa-linkedin:before {
 content:""
}
.fa-rotate-left:before,
.fa-undo:before {
 content:""
}
.fa-legal:before,
.fa-gavel:before {
 content:""
}
.fa-dashboard:before,
.fa-tachometer:before {
 content:""
}
.fa-comment-o:before {
 content:""
}
.fa-comments-o:before {
 content:""
}
.fa-flash:before,
.fa-bolt:before {
 content:""
}
.fa-sitemap:before {
 content:""
}
.fa-umbrella:before {
 content:""
}
.fa-paste:before,
.fa-clipboard:before {
 content:""
}
.fa-lightbulb-o:before {
 content:""
}
.fa-exchange:before {
 content:""
}
.fa-cloud-download:before {
 content:""
}
.fa-cloud-upload:before {
 content:""
}
.fa-user-md:before {
 content:""
}
.fa-stethoscope:before {
 content:""
}
.fa-suitcase:before {
 content:""
}
.fa-bell-o:before {
 content:""
}
.fa-coffee:before {
 content:""
}
.fa-cutlery:before {
 content:""
}
.fa-file-text-o:before {
 content:""
}
.fa-building-o:before {
 content:""
}
.fa-hospital-o:before {
 content:""
}
.fa-ambulance:before {
 content:""
}
.fa-medkit:before {
 content:""
}
.fa-fighter-jet:before {
 content:""
}
.fa-beer:before {
 content:""
}
.fa-h-square:before {
 content:""
}
.fa-plus-square:before {
 content:""
}
.fa-angle-double-left:before {
 content:""
}
.fa-angle-double-right:before {
 content:""
}
.fa-angle-double-up:before {
 content:""
}
.fa-angle-double-down:before {
 content:""
}
.fa-angle-left:before {
 content:""
}
.fa-angle-right:before {
 content:""
}
.fa-angle-up:before {
 content:""
}
.fa-angle-down:before {
 content:""
}
.fa-desktop:before {
 content:""
}
.fa-laptop:before {
 content:""
}
.fa-tablet:before {
 content:""
}
.fa-mobile-phone:before,
.fa-mobile:before {
 content:""
}
.fa-circle-o:before {
 content:""
}
.fa-quote-left:before {
 content:""
}
.fa-quote-right:before {
 content:""
}
.fa-spinner:before {
 content:""
}
.fa-circle:before {
 content:""
}
.fa-mail-reply:before,
.fa-reply:before {
 content:""
}
.fa-github-alt:before {
 content:""
}
.fa-folder-o:before {
 content:""
}
.fa-folder-open-o:before {
 content:""
}
.fa-smile-o:before {
 content:""
}
.fa-frown-o:before {
 content:""
}
.fa-meh-o:before {
 content:""
}
.fa-gamepad:before {
 content:""
}
.fa-keyboard-o:before {
 content:""
}
.fa-flag-o:before {
 content:""
}
.fa-flag-checkered:before {
 content:""
}
.fa-terminal:before {
 content:""
}
.fa-code:before {
 content:""
}
.fa-mail-reply-all:before,
.fa-reply-all:before {
 content:""
}
.fa-star-half-empty:before,
.fa-star-half-full:before,
.fa-star-half-o:before {
 content:""
}
.fa-location-arrow:before {
 content:""
}
.fa-crop:before {
 content:""
}
.fa-code-fork:before {
 content:""
}
.fa-unlink:before,
.fa-chain-broken:before {
 content:""
}
.fa-question:before {
 content:""
}
.fa-info:before {
 content:""
}
.fa-exclamation:before {
 content:""
}
.fa-superscript:before {
 content:""
}
.fa-subscript:before {
 content:""
}
.fa-eraser:before {
 content:""
}
.fa-puzzle-piece:before {
 content:""
}
.fa-microphone:before {
 content:""
}
.fa-microphone-slash:before {
 content:""
}
.fa-shield:before {
 content:""
}
.fa-calendar-o:before {
 content:""
}
.fa-fire-extinguisher:before {
 content:""
}
.fa-rocket:before {
 content:""
}
.fa-maxcdn:before {
 content:""
}
.fa-chevron-circle-left:before {
 content:""
}
.fa-chevron-circle-right:before {
 content:""
}
.fa-chevron-circle-up:before {
 content:""
}
.fa-chevron-circle-down:before {
 content:""
}
.fa-html5:before {
 content:""
}
.fa-css3:before {
 content:""
}
.fa-anchor:before {
 content:""
}
.fa-unlock-alt:before {
 content:""
}
.fa-bullseye:before {
 content:""
}
.fa-ellipsis-h:before {
 content:""
}
.fa-ellipsis-v:before {
 content:""
}
.fa-rss-square:before {
 content:""
}
.fa-play-circle:before {
 content:""
}
.fa-ticket:before {
 content:""
}
.fa-minus-square:before {
 content:""
}
.fa-minus-square-o:before {
 content:""
}
.fa-level-up:before {
 content:""
}
.fa-level-down:before {
 content:""
}
.fa-check-square:before {
 content:""
}
.fa-pencil-square:before {
 content:""
}
.fa-external-link-square:before {
 content:""
}
.fa-share-square:before {
 content:""
}
.fa-compass:before {
 content:""
}
.fa-toggle-down:before,
.fa-caret-square-o-down:before {
 content:""
}
.fa-toggle-up:before,
.fa-caret-square-o-up:before {
 content:""
}
.fa-toggle-right:before,
.fa-caret-square-o-right:before {
 content:""
}
.fa-euro:before,
.fa-eur:before {
 content:""
}
.fa-gbp:before {
 content:""
}
.fa-dollar:before,
.fa-usd:before {
 content:""
}
.fa-rupee:before,
.fa-inr:before {
 content:""
}
.fa-cny:before,
.fa-rmb:before,
.fa-yen:before,
.fa-jpy:before {
 content:""
}
.fa-ruble:before,
.fa-rouble:before,
.fa-rub:before {
 content:""
}
.fa-won:before,
.fa-krw:before {
 content:""
}
.fa-bitcoin:before,
.fa-btc:before {
 content:""
}
.fa-file:before {
 content:""
}
.fa-file-text:before {
 content:""
}
.fa-sort-alpha-asc:before {
 content:""
}
.fa-sort-alpha-desc:before {
 content:""
}
.fa-sort-amount-asc:before {
 content:""
}
.fa-sort-amount-desc:before {
 content:""
}
.fa-sort-numeric-asc:before {
 content:""
}
.fa-sort-numeric-desc:before {
 content:""
}
.fa-thumbs-up:before {
 content:""
}
.fa-thumbs-down:before {
 content:""
}
.fa-youtube-square:before {
 content:""
}
.fa-youtube:before {
 content:""
}
.fa-xing:before {
 content:""
}
.fa-xing-square:before {
 content:""
}
.fa-youtube-play:before {
 content:""
}
.fa-dropbox:before {
 content:""
}
.fa-stack-overflow:before {
 content:""
}
.fa-instagram:before {
 content:""
}
.fa-flickr:before {
 content:""
}
.fa-adn:before {
 content:""
}
.fa-bitbucket:before {
 content:""
}
.fa-bitbucket-square:before {
 content:""
}
.fa-tumblr:before {
 content:""
}
.fa-tumblr-square:before {
 content:""
}
.fa-long-arrow-down:before {
 content:""
}
.fa-long-arrow-up:before {
 content:""
}
.fa-long-arrow-left:before {
 content:""
}
.fa-long-arrow-right:before {
 content:""
}
.fa-apple:before {
 content:""
}
.fa-windows:before {
 content:""
}
.fa-android:before {
 content:""
}
.fa-linux:before {
 content:""
}
.fa-dribbble:before {
 content:""
}
.fa-skype:before {
 content:""
}
.fa-foursquare:before {
 content:""
}
.fa-trello:before {
 content:""
}
.fa-female:before {
 content:""
}
.fa-male:before {
 content:""
}
.fa-gittip:before,
.fa-gratipay:before {
 content:""
}
.fa-sun-o:before {
 content:""
}
.fa-moon-o:before {
 content:""
}
.fa-archive:before {
 content:""
}
.fa-bug:before {
 content:""
}
.fa-vk:before {
 content:""
}
.fa-weibo:before {
 content:""
}
.fa-renren:before {
 content:""
}
.fa-pagelines:before {
 content:""
}
.fa-stack-exchange:before {
 content:""
}
.fa-arrow-circle-o-right:before {
 content:""
}
.fa-arrow-circle-o-left:before {
 content:""
}
.fa-toggle-left:before,
.fa-caret-square-o-left:before {
 content:""
}
.fa-dot-circle-o:before {
 content:""
}
.fa-wheelchair:before {
 content:""
}
.fa-vimeo-square:before {
 content:""
}
.fa-turkish-lira:before,
.fa-try:before {
 content:""
}
.fa-plus-square-o:before {
 content:""
}
.fa-space-shuttle:before {
 content:""
}
.fa-slack:before {
 content:""
}
.fa-envelope-square:before {
 content:""
}
.fa-wordpress:before {
 content:""
}
.fa-openid:before {
 content:""
}
.fa-institution:before,
.fa-bank:before,
.fa-university:before {
 content:""
}
.fa-mortar-board:before,
.fa-graduation-cap:before {
 content:""
}
.fa-yahoo:before {
 content:""
}
.fa-google:before {
 content:""
}
.fa-reddit:before {
 content:""
}
.fa-reddit-square:before {
 content:""
}
.fa-stumbleupon-circle:before {
 content:""
}
.fa-stumbleupon:before {
 content:""
}
.fa-delicious:before {
 content:""
}
.fa-digg:before {
 content:""
}
.fa-pied-piper-pp:before {
 content:""
}
.fa-pied-piper-alt:before {
 content:""
}
.fa-drupal:before {
 content:""
}
.fa-joomla:before {
 content:""
}
.fa-language:before {
 content:""
}
.fa-fax:before {
 content:""
}
.fa-building:before {
 content:""
}
.fa-child:before {
 content:""
}
.fa-paw:before {
 content:""
}
.fa-spoon:before {
 content:""
}
.fa-cube:before {
 content:""
}
.fa-cubes:before {
 content:""
}
.fa-behance:before {
 content:""
}
.fa-behance-square:before {
 content:""
}
.fa-steam:before {
 content:""
}
.fa-steam-square:before {
 content:""
}
.fa-recycle:before {
 content:""
}
.fa-automobile:before,
.fa-car:before {
 content:""
}
.fa-cab:before,
.fa-taxi:before {
 content:""
}
.fa-tree:before {
 content:""
}
.fa-spotify:before {
 content:""
}
.fa-deviantart:before {
 content:""
}
.fa-soundcloud:before {
 content:""
}
.fa-database:before {
 content:""
}
.fa-file-pdf-o:before {
 content:""
}
.fa-file-word-o:before {
 content:""
}
.fa-file-excel-o:before {
 content:""
}
.fa-file-powerpoint-o:before {
 content:""
}
.fa-file-photo-o:before,
.fa-file-picture-o:before,
.fa-file-image-o:before {
 content:""
}
.fa-file-zip-o:before,
.fa-file-archive-o:before {
 content:""
}
.fa-file-sound-o:before,
.fa-file-audio-o:before {
 content:""
}
.fa-file-movie-o:before,
.fa-file-video-o:before {
 content:""
}
.fa-file-code-o:before {
 content:""
}
.fa-vine:before {
 content:""
}
.fa-codepen:before {
 content:""
}
.fa-jsfiddle:before {
 content:""
}
.fa-life-bouy:before,
.fa-life-buoy:before,
.fa-life-saver:before,
.fa-support:before,
.fa-life-ring:before {
 content:""
}
.fa-circle-o-notch:before {
 content:""
}
.fa-ra:before,
.fa-resistance:before,
.fa-rebel:before {
 content:""
}
.fa-ge:before,
.fa-empire:before {
 content:""
}
.fa-git-square:before {
 content:""
}
.fa-git:before {
 content:""
}
.fa-y-combinator-square:before,
.fa-yc-square:before,
.fa-hacker-news:before {
 content:""
}
.fa-tencent-weibo:before {
 content:""
}
.fa-qq:before {
 content:""
}
.fa-wechat:before,
.fa-weixin:before {
 content:""
}
.fa-send:before,
.fa-paper-plane:before {
 content:""
}
.fa-send-o:before,
.fa-paper-plane-o:before {
 content:""
}
.fa-history:before {
 content:""
}
.fa-circle-thin:before {
 content:""
}
.fa-header:before {
 content:""
}
.fa-paragraph:before {
 content:""
}
.fa-sliders:before {
 content:""
}
.fa-share-alt:before {
 content:""
}
.fa-share-alt-square:before {
 content:""
}
.fa-bomb:before {
 content:""
}
.fa-soccer-ball-o:before,
.fa-futbol-o:before {
 content:""
}
.fa-tty:before {
 content:""
}
.fa-binoculars:before {
 content:""
}
.fa-plug:before {
 content:""
}
.fa-slideshare:before {
 content:""
}
.fa-twitch:before {
 content:""
}
.fa-yelp:before {
 content:""
}
.fa-newspaper-o:before {
 content:""
}
.fa-wifi:before {
 content:""
}
.fa-calculator:before {
 content:""
}
.fa-paypal:before {
 content:""
}
.fa-google-wallet:before {
 content:""
}
.fa-cc-visa:before {
 content:""
}
.fa-cc-mastercard:before {
 content:""
}
.fa-cc-discover:before {
 content:""
}
.fa-cc-amex:before {
 content:""
}
.fa-cc-paypal:before {
 content:""
}
.fa-cc-stripe:before {
 content:""
}
.fa-bell-slash:before {
 content:""
}
.fa-bell-slash-o:before {
 content:""
}
.fa-trash:before {
 content:""
}
.fa-copyright:before {
 content:""
}
.fa-at:before {
 content:""
}
.fa-eyedropper:before {
 content:""
}
.fa-paint-brush:before {
 content:""
}
.fa-birthday-cake:before {
 content:""
}
.fa-area-chart:before {
 content:""
}
.fa-pie-chart:before {
 content:""
}
.fa-line-chart:before {
 content:""
}
.fa-lastfm:before {
 content:""
}
.fa-lastfm-square:before {
 content:""
}
.fa-toggle-off:before {
 content:""
}
.fa-toggle-on:before {
 content:""
}
.fa-bicycle:before {
 content:""
}
.fa-bus:before {
 content:""
}
.fa-ioxhost:before {
 content:""
}
.fa-angellist:before {
 content:""
}
.fa-cc:before {
 content:""
}
.fa-shekel:before,
.fa-sheqel:before,
.fa-ils:before {
 content:""
}
.fa-meanpath:before {
 content:""
}
.fa-buysellads:before {
 content:""
}
.fa-connectdevelop:before {
 content:""
}
.fa-dashcube:before {
 content:""
}
.fa-forumbee:before {
 content:""
}
.fa-leanpub:before {
 content:""
}
.fa-sellsy:before {
 content:""
}
.fa-shirtsinbulk:before {
 content:""
}
.fa-simplybuilt:before {
 content:""
}
.fa-skyatlas:before {
 content:""
}
.fa-cart-plus:before {
 content:""
}
.fa-cart-arrow-down:before {
 content:""
}
.fa-diamond:before {
 content:""
}
.fa-ship:before {
 content:""
}
.fa-user-secret:before {
 content:""
}
.fa-motorcycle:before {
 content:""
}
.fa-street-view:before {
 content:""
}
.fa-heartbeat:before {
 content:""
}
.fa-venus:before {
 content:""
}
.fa-mars:before {
 content:""
}
.fa-mercury:before {
 content:""
}
.fa-intersex:before,
.fa-transgender:before {
 content:""
}
.fa-transgender-alt:before {
 content:""
}
.fa-venus-double:before {
 content:""
}
.fa-mars-double:before {
 content:""
}
.fa-venus-mars:before {
 content:""
}
.fa-mars-stroke:before {
 content:""
}
.fa-mars-stroke-v:before {
 content:""
}
.fa-mars-stroke-h:before {
 content:""
}
.fa-neuter:before {
 content:""
}
.fa-genderless:before {
 content:""
}
.fa-facebook-official:before {
 content:""
}
.fa-pinterest-p:before {
 content:""
}
.fa-whatsapp:before {
 content:""
}
.fa-server:before {
 content:""
}
.fa-user-plus:before {
 content:""
}
.fa-user-times:before {
 content:""
}
.fa-hotel:before,
.fa-bed:before {
 content:""
}
.fa-viacoin:before {
 content:""
}
.fa-train:before {
 content:""
}
.fa-subway:before {
 content:""
}
.fa-medium:before {
 content:""
}
.fa-yc:before,
.fa-y-combinator:before {
 content:""
}
.fa-optin-monster:before {
 content:""
}
.fa-opencart:before {
 content:""
}
.fa-expeditedssl:before {
 content:""
}
.fa-battery-4:before,
.fa-battery:before,
.fa-battery-full:before {
 content:""
}
.fa-battery-3:before,
.fa-battery-three-quarters:before {
 content:""
}
.fa-battery-2:before,
.fa-battery-half:before {
 content:""
}
.fa-battery-1:before,
.fa-battery-quarter:before {
 content:""
}
.fa-battery-0:before,
.fa-battery-empty:before {
 content:""
}
.fa-mouse-pointer:before {
 content:""
}
.fa-i-cursor:before {
 content:""
}
.fa-object-group:before {
 content:""
}
.fa-object-ungroup:before {
 content:""
}
.fa-sticky-note:before {
 content:""
}
.fa-sticky-note-o:before {
 content:""
}
.fa-cc-jcb:before {
 content:""
}
.fa-cc-diners-club:before {
 content:""
}
.fa-clone:before {
 content:""
}
.fa-balance-scale:before {
 content:""
}
.fa-hourglass-o:before {
 content:""
}
.fa-hourglass-1:before,
.fa-hourglass-start:before {
 content:""
}
.fa-hourglass-2:before,
.fa-hourglass-half:before {
 content:""
}
.fa-hourglass-3:before,
.fa-hourglass-end:before {
 content:""
}
.fa-hourglass:before {
 content:""
}
.fa-hand-grab-o:before,
.fa-hand-rock-o:before {
 content:""
}
.fa-hand-stop-o:before,
.fa-hand-paper-o:before {
 content:""
}
.fa-hand-scissors-o:before {
 content:""
}
.fa-hand-lizard-o:before {
 content:""
}
.fa-hand-spock-o:before {
 content:""
}
.fa-hand-pointer-o:before {
 content:""
}
.fa-hand-peace-o:before {
 content:""
}
.fa-trademark:before {
 content:""
}
.fa-registered:before {
 content:""
}
.fa-creative-commons:before {
 content:""
}
.fa-gg:before {
 content:""
}
.fa-gg-circle:before {
 content:""
}
.fa-tripadvisor:before {
 content:""
}
.fa-odnoklassniki:before {
 content:""
}
.fa-odnoklassniki-square:before {
 content:""
}
.fa-get-pocket:before {
 content:""
}
.fa-wikipedia-w:before {
 content:""
}
.fa-safari:before {
 content:""
}
.fa-chrome:before {
 content:""
}
.fa-firefox:before {
 content:""
}
.fa-opera:before {
 content:""
}
.fa-internet-explorer:before {
 content:""
}
.fa-tv:before,
.fa-television:before {
 content:""
}
.fa-contao:before {
 content:""
}
.fa-500px:before {
 content:""
}
.fa-amazon:before {
 content:""
}
.fa-calendar-plus-o:before {
 content:""
}
.fa-calendar-minus-o:before {
 content:""
}
.fa-calendar-times-o:before {
 content:""
}
.fa-calendar-check-o:before {
 content:""
}
.fa-industry:before {
 content:""
}
.fa-map-pin:before {
 content:""
}
.fa-map-signs:before {
 content:""
}
.fa-map-o:before {
 content:""
}
.fa-map:before {
 content:""
}
.fa-commenting:before {
 content:""
}
.fa-commenting-o:before {
 content:""
}
.fa-houzz:before {
 content:""
}
.fa-vimeo:before {
 content:""
}
.fa-black-tie:before {
 content:""
}
.fa-fonticons:before {
 content:""
}
.fa-reddit-alien:before {
 content:""
}
.fa-edge:before {
 content:""
}
.fa-credit-card-alt:before {
 content:""
}
.fa-codiepie:before {
 content:""
}
.fa-modx:before {
 content:""
}
.fa-fort-awesome:before {
 content:""
}
.fa-usb:before {
 content:""
}
.fa-product-hunt:before {
 content:""
}
.fa-mixcloud:before {
 content:""
}
.fa-scribd:before {
 content:""
}
.fa-pause-circle:before {
 content:""
}
.fa-pause-circle-o:before {
 content:""
}
.fa-stop-circle:before {
 content:""
}
.fa-stop-circle-o:before {
 content:""
}
.fa-shopping-bag:before {
 content:""
}
.fa-shopping-basket:before {
 content:""
}
.fa-hashtag:before {
 content:""
}
.fa-bluetooth:before {
 content:""
}
.fa-bluetooth-b:before {
 content:""
}
.fa-percent:before {
 content:""
}
.fa-gitlab:before {
 content:""
}
.fa-wpbeginner:before {
 content:""
}
.fa-wpforms:before {
 content:""
}
.fa-envira:before {
 content:""
}
.fa-universal-access:before {
 content:""
}
.fa-wheelchair-alt:before {
 content:""
}
.fa-question-circle-o:before {
 content:""
}
.fa-blind:before {
 content:""
}
.fa-audio-description:before {
 content:""
}
.fa-volume-control-phone:before {
 content:""
}
.fa-braille:before {
 content:""
}
.fa-assistive-listening-systems:before {
 content:""
}
.fa-asl-interpreting:before,
.fa-american-sign-language-interpreting:before {
 content:""
}
.fa-deafness:before,
.fa-hard-of-hearing:before,
.fa-deaf:before {
 content:""
}
.fa-glide:before {
 content:""
}
.fa-glide-g:before {
 content:""
}
.fa-signing:before,
.fa-sign-language:before {
 content:""
}
.fa-low-vision:before {
 content:""
}
.fa-viadeo:before {
 content:""
}
.fa-viadeo-square:before {
 content:""
}
.fa-snapchat:before {
 content:""
}
.fa-snapchat-ghost:before {
 content:""
}
.fa-snapchat-square:before {
 content:""
}
.fa-pied-piper:before {
 content:""
}
.fa-first-order:before {
 content:""
}
.fa-yoast:before {
 content:""
}
.fa-themeisle:before {
 content:""
}
.fa-google-plus-circle:before,
.fa-google-plus-official:before {
 content:""
}
.fa-fa:before,
.fa-font-awesome:before {
 content:""
}
.fa-handshake-o:before {
 content:""
}
.fa-envelope-open:before {
 content:""
}
.fa-envelope-open-o:before {
 content:""
}
.fa-linode:before {
 content:""
}
.fa-address-book:before {
 content:""
}
.fa-address-book-o:before {
 content:""
}
.fa-vcard:before,
.fa-address-card:before {
 content:""
}
.fa-vcard-o:before,
.fa-address-card-o:before {
 content:""
}
.fa-user-circle:before {
 content:""
}
.fa-user-circle-o:before {
 content:""
}
.fa-user-o:before {
 content:""
}
.fa-id-badge:before {
 content:""
}
.fa-drivers-license:before,
.fa-id-card:before {
 content:""
}
.fa-drivers-license-o:before,
.fa-id-card-o:before {
 content:""
}
.fa-quora:before {
 content:""
}
.fa-free-code-camp:before {
 content:""
}
.fa-telegram:before {
 content:""
}
.fa-thermometer-4:before,
.fa-thermometer:before,
.fa-thermometer-full:before {
 content:""
}
.fa-thermometer-3:before,
.fa-thermometer-three-quarters:before {
 content:""
}
.fa-thermometer-2:before,
.fa-thermometer-half:before {
 content:""
}
.fa-thermometer-1:before,
.fa-thermometer-quarter:before {
 content:""
}
.fa-thermometer-0:before,
.fa-thermometer-empty:before {
 content:""
}
.fa-shower:before {
 content:""
}
.fa-bathtub:before,
.fa-s15:before,
.fa-bath:before {
 content:""
}
.fa-podcast:before {
 content:""
}
.fa-window-maximize:before {
 content:""
}
.fa-window-minimize:before {
 content:""
}
.fa-window-restore:before {
 content:""
}
.fa-times-rectangle:before,
.fa-window-close:before {
 content:""
}
.fa-times-rectangle-o:before,
.fa-window-close-o:before {
 content:""
}
.fa-bandcamp:before {
 content:""
}
.fa-grav:before {
 content:""
}
.fa-etsy:before {
 content:""
}
.fa-imdb:before {
 content:""
}
.fa-ravelry:before {
 content:""
}
.fa-eercast:before {
 content:""
}
.fa-microchip:before {
 content:""
}
.fa-snowflake-o:before {
 content:""
}
.fa-superpowers:before {
 content:""
}
.fa-wpexplorer:before {
 content:""
}
.fa-meetup:before {
 content:""
}
.sr-only {
 position:absolute;
 width:1px;
 height:1px;
 padding:0;
 margin:-1px;
 overflow:hidden;
 clip:rect(0, 0, 0, 0);
 border:0
}
.sr-only-focusable:active,
.sr-only-focusable:focus {
 position:static;
 width:auto;
 height:auto;
 margin:0;
 overflow:visible;
 clip:auto
}
/*!
 * animate.css -http://daneden.me/animate
 * Version - 3.5.1
 * Licensed under the MIT license - http://opensource.org/licenses/MIT
 *
 * Copyright (c) 2016 Daniel Eden
 */.animated {
 -webkit-animation-duration:1s;
 animation-duration:1s;
 -webkit-animation-fill-mode:both;
 animation-fill-mode:both
}
@-webkit-keyframes fadeIn {
 from {
  opacity:0
 }
 to {
  opacity:1
 }
}
@keyframes fadeIn {
 from {
  opacity:0
 }
 to {
  opacity:1
 }
}
.fadeIn {
 -webkit-animation-name:fadeIn;
 animation-name:fadeIn
}
@-webkit-keyframes fadeInDown {
 from {
  opacity:0;
  -webkit-transform:translate3d(0, -100%, 0);
  transform:translate3d(0, -100%, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
@keyframes fadeInDown {
 from {
  opacity:0;
  -webkit-transform:translate3d(0, -100%, 0);
  transform:translate3d(0, -100%, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
.fadeInDown {
 -webkit-animation-name:fadeInDown;
 animation-name:fadeInDown
}
@-webkit-keyframes fadeInLeft {
 from {
  opacity:0;
  -webkit-transform:translate3d(-100%, 0, 0);
  transform:translate3d(-100%, 0, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
@keyframes fadeInLeft {
 from {
  opacity:0;
  -webkit-transform:translate3d(-100%, 0, 0);
  transform:translate3d(-100%, 0, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
.fadeInLeft {
 -webkit-animation-name:fadeInLeft;
 animation-name:fadeInLeft
}
@-webkit-keyframes fadeInRight {
 from {
  opacity:0;
  -webkit-transform:translate3d(100%, 0, 0);
  transform:translate3d(100%, 0, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
@keyframes fadeInRight {
 from {
  opacity:0;
  -webkit-transform:translate3d(100%, 0, 0);
  transform:translate3d(100%, 0, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
.fadeInRight {
 -webkit-animation-name:fadeInRight;
 animation-name:fadeInRight
}
@-webkit-keyframes fadeInRightBig {
 from {
  opacity:0;
  -webkit-transform:translate3d(2000px, 0, 0);
  transform:translate3d(2000px, 0, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
@keyframes fadeInUp {
 from {
  opacity:0;
  -webkit-transform:translate3d(0, 100%, 0);
  transform:translate3d(0, 100%, 0)
 }
 to {
  opacity:1;
  -webkit-transform:none;
  transform:none
 }
}
.fadeInUp {
 -webkit-animation-name:fadeInUp;
 animation-name:fadeInUp
}
@-webkit-keyframes fadeOut {
 from {
  opacity:1
 }
 to {
  opacity:0
 }
}
@keyframes fadeOut {
 from {
  opacity:1
 }
 to {
  opacity:0
 }
}
.fadeOut {
 -webkit-animation-name:fadeOut;
 animation-name:fadeOut
}
@-webkit-keyframes rotateIn {
 from {
  -webkit-transform-origin:center;
  transform-origin:center;
  -webkit-transform:rotate3d(0, 0, 1, -200deg);
  transform:rotate3d(0, 0, 1, -200deg);
  opacity:0
 }
 to {
  -webkit-transform-origin:center;
  transform-origin:center;
  -webkit-transform:none;
  transform:none;
  opacity:1
 }
}
@keyframes rotateIn {
 from {
  -webkit-transform-origin:center;
  transform-origin:center;
  -webkit-transform:rotate3d(0, 0, 1, -200deg);
  transform:rotate3d(0, 0, 1, -200deg);
  opacity:0
 }
 to {
  -webkit-transform-origin:center;
  transform-origin:center;
  -webkit-transform:none;
  transform:none;
  opacity:1
 }
}
.rotateIn {
 -webkit-animation-name:rotateIn;
 animation-name:rotateIn
}
.mfp-counter {
 font-family:serif
}
.mfp-bg {
 top:0;
 left:0;
 width:100%;
 height:100%;
 z-index:1042;
 overflow:hidden;
 position:fixed;
 background:#000;
 opacity:.8
}
.mfp-wrap {
 top:0;
 left:0;
 width:100%;
 height:100%;
 z-index:1043;
 position:fixed;
 outline:none !important;
 -webkit-backface-visibility:hidden
}
.mfp-container {
 text-align:center;
 position:absolute;
 width:100%;
 height:100%;
 left:0;
 top:0;
 padding:0 8px;
 box-sizing:border-box
}
.mfp-container:before {
 content:'';
 display:inline-block;
 height:100%;
 vertical-align:middle
}
.mfp-align-top .mfp-container:before {
 display:none
}
.mfp-content {
 position:relative;
 display:inline-block;
 vertical-align:middle;
 margin:0 auto;
 text-align:left;
 z-index:1045
}
.mfp-inline-holder .mfp-content,
.mfp-ajax-holder .mfp-content {
 width:100%;
 cursor:auto
}
.mfp-ajax-cur {
 cursor:progress
}
.mfp-zoom-out-cur,
.mfp-zoom-out-cur .mfp-image-holder .mfp-close {
 cursor:-moz-zoom-out;
 cursor:-webkit-zoom-out;
 cursor:zoom-out
}
.mfp-zoom {
 cursor:pointer;
 cursor:-webkit-zoom-in;
 cursor:-moz-zoom-in;
 cursor:zoom-in
}
.mfp-auto-cursor .mfp-content {
 cursor:auto
}
.mfp-close,
.mfp-arrow,
.mfp-preloader,
.mfp-counter {
 -webkit-user-select:none;
 -moz-user-select:none;
 user-select:none
}
.mfp-loading.mfp-figure {
 display:none
}
.mfp-hide {
 display:none !important
}
.mfp-preloader {
 color:#CCC;
 position:absolute;
 top:50%;
 width:auto;
 text-align:center;
 margin-top:-0.8em;
 left:8px;
 right:8px;
 z-index:1044
}
.mfp-preloader a {
 color:#CCC
}
.mfp-preloader a:hover {
 color:#fff
}
.mfp-s-ready .mfp-preloader {
 display:none
}
.mfp-s-error .mfp-content {
 display:none
}
button.mfp-close,
button.mfp-arrow {
 overflow:visible;
 cursor:pointer;
 background:transparent;
 border:0;
 -webkit-appearance:none;
 display:block;
 outline:none;
 padding:0;
 z-index:1046;
 box-shadow:none;
 touch-action:manipulation
}
button::-moz-focus-inner {
 padding:0;
 border:0
}
.mfp-close {
 width:44px;
 height:44px;
 line-height:44px;
 position:absolute;
 right:0;
 top:0;
 text-decoration:none;
 text-align:center;
 opacity:1;
 padding:0 0 18px 10px;
 color:#fff;
 font-style:normal;
 font-size:28px;
 font-family:Arial, Baskerville, monospace
}
.mfp-close:hover,
.mfp-close:focus {
 opacity:1
}
.mfp-close:active {
 top:1px
}
.mfp-close-btn-in .mfp-close {
 color:#fff
}
.mfp-image-holder .mfp-close,
.mfp-iframe-holder .mfp-close {
 color:#fff;
 right:-6px;
 text-align:right;
 padding-right:6px;
 width:100%
}
.mfp-counter {
 position:absolute;
 top:0;
 right:0;
 color:#CCC;
 font-size:12px;
 line-height:18px;
 white-space:nowrap
}
.mfp-arrow {
 position:absolute;
 opacity:1;
 margin:0;
 top:50%;
 margin-top:-55px;
 padding:0;
 width:90px;
 height:110px;
 -webkit-tap-highlight-color:transparent
}
.mfp-arrow:active {
 margin-top:-54px
}
.mfp-arrow:hover,
.mfp-arrow:focus {
 opacity:1
}
.mfp-arrow:before,
.mfp-arrow:after {
 content:'';
 display:block;
 width:0;
 height:0;
 position:absolute;
 left:0;
 top:0;
 margin-top:35px;
 margin-left:35px;
 border:medium inset transparent
}
.mfp-arrow:after {
 border-top-width:13px;
 border-bottom-width:13px;
 top:8px
}
.mfp-arrow:before {
 border-top-width:21px;
 border-bottom-width:21px;
 opacity:0.7
}
.mfp-arrow-left {
 left:0
}
.mfp-arrow-left:after {
 border-right:17px solid #fff;
 margin-left:31px
}
.mfp-arrow-left:before {
 margin-left:25px;
 border-right:27px solid #fff
}
.mfp-arrow-right {
 right:0
}
.mfp-arrow-right:after {
 border-left:17px solid #fff;
 margin-left:39px
}
.mfp-arrow-right:before {
 border-left:27px solid #fff
}
.mfp-iframe-holder {
 padding-top:40px;
 padding-bottom:40px
}
.mfp-iframe-holder .mfp-content {
 line-height:0;
 width:100%;
 max-width:900px
}
.mfp-iframe-holder .mfp-close {
 top:-40px
}
.mfp-iframe-scaler {
 width:100%;
 height:0;
 overflow:hidden;
 padding-top:56.25%
}
.mfp-iframe-scaler iframe {
 position:absolute;
 display:block;
 top:0;
 left:0;
 width:100%;
 height:100%;
 box-shadow:0 0 8px rgba(0,0,0,0.6);
 background:#000
}
img.mfp-img {
 width:auto;
 max-width:100%;
 height:auto;
 display:block;
 line-height:0;
 box-sizing:border-box;
 padding:40px 0 40px;
 margin:0 auto
}
.mfp-figure {
 line-height:0
}
.mfp-figure:after {
 content:'';
 position:absolute;
 left:0;
 top:40px;
 bottom:40px;
 display:block;
 right:0;
 width:auto;
 height:auto;
 z-index:-1;
 box-shadow:0 0 8px rgba(0,0,0,0.6);
 background:#444
}
.mfp-figure small {
 color:#BDBDBD;
 display:block;
 font-size:12px;
 line-height:14px
}
.mfp-figure figure {
 margin:0
}
.mfp-bottom-bar {
 margin-top:-36px;
 position:absolute;
 top:100%;
 left:0;
 width:100%;
 cursor:auto
}
.mfp-title {
 text-align:left;
 line-height:18px;
 color:#F3F3F3;
 word-wrap:break-word;
 padding-right:36px
}
.mfp-image-holder .mfp-content {
 max-width:100%
}
.mfp-gallery .mfp-image-holder .mfp-figure {
 cursor:pointer
}
@media screen and (max-width: 800px) and (orientation: landscape), screen and (max-height: 300px) {
 .mfp-img-mobile .mfp-image-holder {
  padding-left:0;
  padding-right:0
 }
 .mfp-img-mobile img.mfp-img {
  padding:0
 }
 .mfp-img-mobile .mfp-figure:after {
  top:0;
  bottom:0
 }
 .mfp-img-mobile .mfp-figure small {
  display:inline;
  margin-left:5px
 }
 .mfp-img-mobile .mfp-bottom-bar {
  background:rgba(0,0,0,0.6);
  bottom:0;
  margin:0;
  top:auto;
  padding:3px 5px;
  position:fixed;
  box-sizing:border-box
 }
 .mfp-img-mobile .mfp-bottom-bar:empty {
  padding:0
 }
 .mfp-img-mobile .mfp-counter {
  right:5px;
  top:3px
 }
 .mfp-img-mobile .mfp-close {
  top:0;
  right:0;
  width:35px;
  height:35px;
  line-height:35px;
  background:rgba(0,0,0,0.6);
  position:fixed;
  text-align:center;
  padding:0
 }
}
@media all and (max-width: 900px) {
 .mfp-arrow {
  -webkit-transform:scale(0.75);
  transform:scale(0.75)
 }
 .mfp-arrow-left {
  -webkit-transform-origin:0;
  transform-origin:0
 }
 .mfp-arrow-right {
  -webkit-transform-origin:100%;
  transform-origin:100%
 }
 .mfp-container {
  padding-left:6px;
  padding-right:6px
 }
}
@media print {
 body {
  background-image:none !important;
  background-color:#fff
 }
 .wrapper {
  box-shadow:none;
  width:100%;
  max-width:100%;
  margin:0 auto
 }
 .container {
  box-shadow:none
 }
 .feature {
  padding:0
 }
 .social-btn,
 .btn {
  border-color:#d0d1d5
 }
 .post-title .btn,
 .reading-time,
 #dl-menu,
 #disqus_thread,
 .goup-container {
  display:none
 }
}
