<!DOCTYPE html>
<html>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <link rel="stylesheet" type="text/css" href="{{ css_style_sheet }}" />
    <link rel="stylesheet" type="text/css" href="{{ custom_style }}" />
    <script src="https://polyfill.io/v3/polyfill.min.js?features=es6"></script>
    <script id="MathJax-script" async src="{{ mathjax }}"></script>

    <head>
        <title>{{ group_name }} results</title>
    </head>
    <body>
        <div class="box">
            <div class="container">
                <div class="row">
                    <h1 class="title">FitBenchmarking results: {{ group_name }}</h1>
                    <hr>
                </div>
                <div class="row">
                    <p>FitBenchmarking is a tool for comparing different fitting software based on their accuracy and runtimes.
                    </p>
                </div>
                <h4>Table formats</h4>
                {{ index }}
                <hr>
                <div class="row">
                    {% for type, link, description in zip(table_type, links, description) %}
                        <h3>
                        <a href="{{ link }}"><code>{{ type }}</code> table</a>
                        </h3>
                        <p>{{ description }}</p>
                        <h3>
                        <form>
                            <button class="btn default" type="submit" formaction="{{ link }}">
                            <i class="fa fa-table"></i>
                            </button>
                        </form>
                        </h3>
                        <hr>
                    {% endfor %}
                </div>
                <center>
                <button class="btn default" onclick="history.go(-1)">
                <i class="fa fa-arrow-left"></i>
                </button>
                </center>
            </div>
        </div>
    </body>
</html>
