<!DOCTYPE html>
<html>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <link rel="stylesheet" type="text/css" href="{{ css_style_sheet }}" />
    <link rel="stylesheet" type="text/css" href="{{ custom_style }}" />
    <head>
        <title> Index page</title>
    </head>
    <body>
        <div class="box">
            <div class="container">
                <div class="post-title">
                    <h1>FitBenchmarking results page</h1>
                    <hr />
                </div>
                <p>
                    Follow the links below to the results for the individual groups
                </p>
                {% for group, link in zip(groups, group_link) %}
                    <h2>
                    <a href="{{ link }}"><code>{{ group }}</code></a>
                    </h2>
                {% endfor %}
            </div>
        </div>
    </body>
</html>
